#!/bin/sh

verbose=false

if [ "x$1" = "x-v" ]
then
	verbose=true
	out=/dev/stdout
	err=/dev/stderr
else
	out=/dev/null
	err=/dev/null
fi

pts=5
timeout=30

runbochs () {
	# Find the address of the kernel readline function,
	# which the kernel monitor uses to read commands interactively.
	brkaddr=`grep 'readline$' obj/kern/kernel.sym | sed -e's/ .*$//g'`
	#echo "brkaddr $brkaddr"

	readline_hack=`grep 'readline_hack$' obj/kern/kernel.sym | sed -e's/ .*$//g' | sed -e's/^f//'`

	# Run Bochs, setting a breakpoint at readline(),
	# and feeding in appropriate commands to run, then quit.
	(
		echo vbreak 0x8:0x$brkaddr
		echo c
		echo setpmem 0x$readline_hack 1 1
		echo c
		echo die
		echo quit
	) | (
		ulimit -t $timeout
		bochs -q 'display_library: nogui' \
			'parport1: enabled=1, file="bochs.out"'
	) >$out 2>$err
}



gmake
runbochs

score=0

awk 'BEGIN{printf("Page directory: ");}' </dev/null	# goddamn suns can't echo -n.
 if grep "check_boot_pgdir() succeeded!" bochs.out >/dev/null
 then
	score=`expr 20 + $score`
	echo OK
 else
	echo WRONG
 fi

awk 'BEGIN{printf("Page management: ");}' </dev/null
 if grep "page_check() succeeded!" bochs.out >/dev/null
 then
	score=`expr 30 + $score`
	echo OK
 else
	echo WRONG
 fi

awk 'BEGIN{printf("Kernel breakpoint interrupt: ");}' </dev/null
 if grep "^TRAP frame at 0x" bochs.out >/dev/null \
     && grep "  trap 0x00000003 Breakpoint" bochs.out >/dev/null
 then
	score=`expr 10 + $score`
	echo OK
 else
	echo WRONG
 fi

awk 'BEGIN{printf("Returning from breakpoint interrupt: ");}' </dev/null
 if grep "Breakpoint succeeded!" bochs.out >/dev/null
 then
	score=`expr 10 + $score`
	echo OK
 else
	echo WRONG
 fi

echo "Score: $score/70"


